/*  Somatic - A soma puzzle solver.
 *  Copyright (C) 2003 Marc Moerig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "sGFX_pieces.hpp"
#include "sFOX_piecewin.hpp"

//**************************************************************************

FXDEFMAP(SomaPieceWin) SomaPieceWinMap[]={
  
  //________Message_Type_________ID_____________________Message_Handler_______
  FXMAPFUNC(SEL_PAINT,            SomaPieceWin::ID_GLCANVAS,    SomaPieceWin::onExpose),
  FXMAPFUNC(SEL_CONFIGURE,        SomaPieceWin::ID_GLCANVAS,    SomaPieceWin::onConfigure),
  FXMAPFUNC(SEL_LEFTBUTTONPRESS,  SomaPieceWin::ID_GLCANVAS,    SomaPieceWin::onMouseClick),
  FXMAPFUNC(SEL_RIGHTBUTTONPRESS, SomaPieceWin::ID_GLCANVAS,    SomaPieceWin::onMouseClick),
  FXMAPFUNC(SEL_MIDDLEBUTTONPRESS,SomaPieceWin::ID_GLCANVAS,    SomaPieceWin::onMouseClick),
  FXMAPFUNC(SEL_MOTION,           SomaPieceWin::ID_GLCANVAS,    SomaPieceWin::onMouseMove),
  FXMAPFUNC(SEL_MOUSEWHEEL,       SomaPieceWin::ID_GLCANVAS,    SomaPieceWin::onMouseWheel)
  
}; 

//**************************************************************************

FXIMPLEMENT(SomaPieceWin,FXDialogBox,SomaPieceWinMap,ARRAYNUMBER(SomaPieceWinMap))

//**************************************************************************

// Construct a dialog box
SomaPieceWin::SomaPieceWin(FXWindow* owner,const SomaProb *const p,FXGLCanvas *const other):
               FXDialogBox(owner,"Pieces",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,400,300,0,0,0,0,0,0){
  

  
  FXVerticalFrame *frame = new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED,0,0,0,0,0,0,0,0);
  FXVerticalFrame *glcanvasFrame = new FXVerticalFrame(frame,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,5,5,5,1);

  FXHorizontalFrame *hframe = new FXHorizontalFrame(frame,LAYOUT_FILL_X,0,0,0,0,0,0,0,0);
  FXVerticalFrame *btframe = new FXVerticalFrame(hframe,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,5,5,0,5);
  FXButton *bt = new FXButton(btframe,"Close",NULL,this,ID_HIDE,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X,0,0,0,0,2,2,2,2);
  bt->setHelpText("Close window.");
  
  new FXDragCorner(hframe);
  
  // A Visual to drag OpenGL
  glvisual=new FXGLVisual(getApp(),VISUAL_DOUBLEBUFFER);
  
  // Drawing glcanvas
  glcanvas=new FXGLCanvas(glcanvasFrame,glvisual,other,this,ID_GLCANVAS,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP,0,0,0,0);
  FXColor bc = glcanvasFrame->getBackColor();
  SomaVector<double> sc(FXREDVAL(bc)/255.,FXGREENVAL(bc)/255.,FXBLUEVAL(bc)/255.);
  p_gfx = new SomaPiecesGFX(p,sc);
}

//**************************************************************************

void SomaPieceWin::create(){
  FXDialogBox::create();
  show(PLACEMENT_SCREEN);

  // init gl stuff
  if(glcanvas->makeCurrent()){
    SomaBaseGFX::glInit();
    glcanvas->makeNonCurrent();
  }
}

//**************************************************************************

long SomaPieceWin::onExpose(FXObject*,FXSelector,void*){
  drawScene();
  return 1;
}

//**************************************************************************

long SomaPieceWin::onConfigure(FXObject*,FXSelector,void*){
  if(glcanvas->makeCurrent()){ 
    p_gfx->resize(glcanvas->getWidth(),glcanvas->getHeight());
    glcanvas->makeNonCurrent();
  }
  
  return 1;
}

//**************************************************************************

long SomaPieceWin::onMouseClick(FXObject *,FXSelector selector,void* ptr){
  FXEvent *e = (FXEvent*)ptr;

  SomaBaseGFX::MOUSEBUTTON button = SomaBaseGFX::NONE;

  switch(SELTYPE(selector)){
    case SEL_LEFTBUTTONPRESS:
      button = SomaBaseGFX::LEFT;
      break;
    case SEL_MIDDLEBUTTONPRESS:
      button = SomaBaseGFX::MIDDLE;
      break;
    case SEL_RIGHTBUTTONPRESS:
      button = SomaBaseGFX::RIGHT;
      break;
  }
  
  if(glcanvas->makeCurrent()){
    p_gfx->mouseClick(e->win_x,e->win_y,button);
    glcanvas->makeNonCurrent();
  }
  
  drawScene();
  
  return 1; 
}
//**************************************************************************

long SomaPieceWin::onMouseMove(FXObject*,FXSelector,void* ptr){
  FXEvent *e = (FXEvent*)ptr;

  SomaBaseGFX::MOUSEBUTTON button = SomaBaseGFX::NONE;
  
  
  if(e->state & RIGHTBUTTONMASK) button = SomaBaseGFX::RIGHT;
  else if(e->state & MIDDLEBUTTONMASK) button = SomaBaseGFX::MIDDLE;
  else if(e->state & LEFTBUTTONMASK) button = SomaBaseGFX::LEFT;
  
  if(button != SomaBaseGFX::NONE){
    
    if(glcanvas->makeCurrent()){
      p_gfx->mouseMove(e->win_x - e->last_x,e->win_y - e->last_y,button);
      glcanvas->makeNonCurrent();
    }
   
    drawScene();
  }
  
 
  
  return 1;
}

//**************************************************************************

long SomaPieceWin::onMouseWheel(FXObject*,FXSelector,void* ptr){
  FXEvent *e = (FXEvent*) ptr;
  
  p_gfx->mouseMove(-e->code/30,-e->code/30,SomaBaseGFX::MIDDLE);
  
  drawScene();
  return 1;
}

//**************************************************************************

void SomaPieceWin::drawScene(){
  if(glcanvas->makeCurrent()){
    p_gfx->draw();
    if(glvisual->isDoubleBuffer()) glcanvas->swapBuffers();
    glcanvas->makeNonCurrent();
  }
}

//**************************************************************************

SomaPieceWin::~SomaPieceWin(){
  delete p_gfx;
}

//**************************************************************************
