/*  Somatic - A soma puzzle solver.
 *  Copyright (C) 2003 Marc Moerig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
#include "sGFX_cubearray.hpp"

//**************************************************************************

SomaCubeArray::SomaCubeArray(const unsigned int capacity_in):capacity(capacity_in){
  csize = 0;
  cstack = new SomaCube*[capacity];
  
  for(unsigned int i=0;i<capacity;i++) cstack[i] = 0; 
}

//**************************************************************************

SomaCubeArray::SomaCubeArray(const SomaCubeArray& other):capacity(other.capacity){
  csize = other.csize;
  cstack = new SomaCube*[capacity];
  
  for(unsigned int i=0;i<capacity;i++){
    if(other.cstack[i]){
      cstack[i] = new SomaCube(*other.cstack[i]);
    }else{
      cstack[i] = 0;
    }
  }
}

//**************************************************************************

SomaCubeArray::~SomaCubeArray(){
  for(unsigned int i=0;i<csize;i++){
    delete cstack[i]; 
    cstack[i] = 0;
  } 
  delete[] cstack;
}


//**************************************************************************

void SomaCubeArray::insert(const SomaVector<double>& p,const SomaVector<double>& c){ 
#ifdef SOMA_DEBUG
  if(capacity == csize){
    std::cerr << "Capacity exhausted in SomaCubeArray::insert()." << std::endl;
    return;
  }
#endif   
  
  if (csize < capacity) { 
    cstack[csize] = new SomaCube(p,c);    
    csize++;  
  }
}

//**************************************************************************

void SomaCubeArray::remove(unsigned int pos){
#ifdef SOMA_DEBUG
  if(pos >= capacity){
    std::cerr << "Position out of range in SomaCubeArray::remove()." << std::endl;
    return;
  }
#endif 
  
  if (pos < csize) { 
    
    delete cstack[pos];
    
    while ((pos+1) < csize) {
      cstack[pos] = cstack[pos+1];
      pos++;
    }  
    
    cstack[pos] = 0;
    csize--;
  }
}

//**************************************************************************

void SomaCubeArray::clear(){
  for(unsigned int i=0;i<csize;i++){
    delete cstack[i]; 
    cstack[i] = 0;
  }
  csize = 0;     
}

//**************************************************************************

const SomaCube * SomaCubeArray::get(const unsigned int pos){
#ifdef SOMA_DEBUG
  if(pos >= capacity){
    std::cerr << "Position out of range in SomaCubeArray::get()." << std::endl;
    return 0;
  }
#endif  
  
  return cstack[pos];
}

//**************************************************************************

