/*  Somatic - A soma puzzle solver.
 *  Copyright (C) 2003 Marc Moerig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef soma_gfx_point 
#define soma_gfx_point

#include "s_vector.hpp"

//**************************************************************************

class SomaPoint{
  public:
  
  /** point coordinates */
  SomaVector<double> p;
  
  /** normal for the point */
  SomaVector<double> n;
  
  /** constructor */
  inline SomaPoint(){  
    p.set(0,0,0);
    n.set(0,0,0);
  }
  
  /** constructor */  
  inline SomaPoint(const double px, const double py, const double pz){
    p.set(px,py,pz);  
    n.set(0,0,0);
  }
  
  /** constructor */  
  inline SomaPoint(const double px, const double py, const double pz,
                   const double nx, const double ny, const double nz){
    p.set(px,py,pz);  
    n.set(nx,ny,nz);
  }
  
  /** constructor */  
  inline SomaPoint(const SomaVector<double>& p){
    this->p = p;
    n.set(0,0,0);
  }
  
  /** constructor */  
  inline SomaPoint(const SomaVector<double>& p,const SomaVector<double>& n){
    this->p = p;
    this->n = n;
  }
  
  /** destructor */
  inline ~SomaPoint(){};
};

//**************************************************************************

/** create a normal for a triangle */
inline SomaVector<double> normal(const SomaPoint &p1, 
                                 const SomaPoint &p2, const SomaPoint &p3){
  return cross((p2.p - p1.p),  (p3.p - p1.p));
}

//**************************************************************************

/** bisection of two points */
inline SomaPoint bisection(const SomaPoint &p1, const SomaPoint &p2){
  SomaPoint p((p1.p + p2.p)/2.,(p1.n + p2.n)/2.);
  return p;
}

//**************************************************************************

#endif

