/*  Somatic - A soma puzzle solver.
 *  Copyright (C) 2003 Marc Moerig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "sGFX_solution.hpp"
 
#include "s_piece.hpp"
#include "sGFX_colorchooser.hpp"
#include "sGFX_cube.hpp"

//**************************************************************************

SomaSolGFX::SomaSolGFX(SomaProb *const p,const SomaSingleGFX *const gfx):
                       SomaSingleGFX(p,gfx){
  scale = 1;
  solution = 0;   
  initCubes();
}

//**************************************************************************

SomaSolGFX::SomaSolGFX(SomaProb *const p):SomaSingleGFX(p){
  scale = 1;
  solution = 0;
  initCubes();
}

//**************************************************************************

SomaSolGFX::~SomaSolGFX(){}

//**************************************************************************

void SomaSolGFX::initCubes(){
  
  cubes.clear();
  
  SomaColorChooser rgb(pbl->get_num_pieces());
  
  SomaVector<int> bpos,ppos;
  SomaVector<int> poff;
  unsigned int pori;
  SomaPiece piece(1);
  
  SomaVector<double> foff(pbl->center());

  for(unsigned int tcnt=0;tcnt < pbl->get_num_pieces();tcnt++){

    piece.set(*pbl->get_piece(tcnt));

    if (pbl->get_solution()->get(solution)->get(tcnt,&ppos,&pori)){
      piece.set_orientation(pori);
      SomaVector<double> poff(piece.get_offset());
      for(unsigned int bcnt=0;bcnt< piece.get_num_blocks();bcnt++){
    
        piece.get(bcnt,&bpos);
        SomaVector<double> gfxpos = (foff+SomaVector<double>(ppos)-poff)*scale+poff+SomaVector<double>(bpos);
        cubes.insert(gfxpos,rgb.get(tcnt));     
      } 
    }  
  }
}

//**************************************************************************

void SomaSolGFX::display(const GLenum mode){
  if (mode == GL_RENDER){
    for(unsigned int i=0;i < cubes.size();i++){
      cubes.get(i)->draw(mode);  
    }
  }
}

//**************************************************************************

void SomaSolGFX::mouseClick(const unsigned int x,const unsigned int y,
                            const MOUSEBUTTON button){

}

//**************************************************************************

void SomaSolGFX::mouseMove(const int deltax,const int deltay,
                           const MOUSEBUTTON button){
   if(button == MIDDLE){
    dist += 0.2 * deltay; 
    if(dist < zoomstart-zoomrange) dist = zoomstart-zoomrange;
    if(dist > zoomstart+zoomrange) dist = zoomstart+zoomrange; 
  }else if(button == LEFT){
    rotation.rotate(SomaMatrix::Y,0.015*deltax);
    rotation.rotate(SomaMatrix::X,0.015*deltay);
  }else if(button == RIGHT){
    scale += 0.015*deltax;
  
    if (scale < 1 ) scale = 1;
    if (scale > 4 ) scale = 4;

    initCubes();  
  }
}

//**************************************************************************

void SomaSolGFX::nextSolution(){
  solution++;
  if(solution == pbl->get_solution()->size()) solution = 0;
  initCubes();
}

//**************************************************************************

void SomaSolGFX::prevSolution(){
  if(solution == 0) solution = pbl->get_solution()->size()-1;
  else solution--;
  initCubes();
}

//**************************************************************************

