/*  Somatic - A soma puzzle solver.
 *  Copyright (C) 2003 Marc Moerig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef soma_io
#define soma_io

#include <iostream>

//**************************************************************************

/** A base class for handling input/output. */
class SomaIO{
  public:
  virtual ~SomaIO() { };
  
  /** Write object information on a stream. */
  virtual void write(std::ostream& strm = std::cout) const = 0;
  
  /** Read object information from a stream. 
  Returns true if the object could be read successfully. */
  virtual bool read(std::istream& strm = std::cin) = 0;
};

/** The operator for reading a SomaIO object. */
inline std::ostream& operator << (std::ostream& strm,const SomaIO& IO){
  IO.write(strm);
  return strm;
}

/** The operator for writing a SomaIO object. */
inline std::istream& operator >> (std::istream& strm,SomaIO& IO){
  IO.read(strm);
  return strm;
}

//**************************************************************************

#endif

