/*  Somatic - A soma puzzle solver.
 *  Copyright (C) 2003 Marc Moerig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "s_piece.hpp"

//**************************************************************************

const unsigned int SomaPiece::ORIENTATIONS = 24;

//**************************************************************************

const unsigned int lookup[24][6] = {{ 8,12,18,20, 3, 1},
                                    {11,13,17,21, 0, 2},
                                    {10,14,16,22, 1, 3},
                                    { 9,15,19,23, 2, 0},
                                    
                                    {12, 8,22,16, 7, 5},
                                    {15, 9,21,17, 4, 6},
                                    {14,10,20,18, 5, 7},
                                    {13,11,23,19, 6, 4},
  
                                    { 4, 0,11, 9,19,21},
                                    { 5, 3, 8,10,16,20},
                                    { 6, 2, 9,11,17,23},
                                    { 7, 1,10, 8,18,22},
                                    
                                    { 0, 4,15,13,23,17},
                                    { 1, 7,12,14,20,16},
                                    { 2, 6,13,15,21,19},
                                    { 3, 5,14,12,22,18},
                                    
                                    {17,19, 4, 2,13, 9},
                                    {18,16, 5, 1,12,10},
                                    {19,17, 6, 0,15,11},
                                    {16,18, 7, 3,14, 8},
                                    
                                    {21,23, 0, 6, 9,13},
                                    {22,20, 1, 5, 8,14},
                                    {23,21, 2, 4,11,15},
                                    {20,22, 3, 7,10,12}};

//**************************************************************************
                                    
const SomaPiece::DIRECTION set_pos_lookup[24][24]={
{SomaPiece::NO,SomaPiece::YP,SomaPiece::YM,SomaPiece::YM,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::XP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZP,
 SomaPiece::XP,SomaPiece::ZM,SomaPiece::XM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP},
{SomaPiece::YM,SomaPiece::NO,SomaPiece::YP,SomaPiece::YM,SomaPiece::ZM,SomaPiece::XP,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::XP,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::XM,SomaPiece::ZM,SomaPiece::ZM},
{SomaPiece::YM,SomaPiece::YM,SomaPiece::NO,SomaPiece::YP,SomaPiece::XP,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZP,SomaPiece::XP,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::XM,SomaPiece::ZM},
{SomaPiece::YP,SomaPiece::YM,SomaPiece::YM,SomaPiece::NO,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::XP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZP,SomaPiece::XP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::XM},
{SomaPiece::ZM,SomaPiece::ZM,SomaPiece::XP,SomaPiece::ZM,SomaPiece::NO,SomaPiece::YP,
 SomaPiece::YM,SomaPiece::YM,SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZP,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::XM,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::XP,SomaPiece::ZM},
{SomaPiece::ZM,SomaPiece::XP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::YM,SomaPiece::NO,
 SomaPiece::YP,SomaPiece::YM,SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::XM,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::XP,SomaPiece::ZM,SomaPiece::ZM},
{SomaPiece::XP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::YM,SomaPiece::YM,
 SomaPiece::NO,SomaPiece::YP,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZP,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,
 SomaPiece::XM,SomaPiece::ZM,SomaPiece::XP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP},
{SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::XP,SomaPiece::YP,SomaPiece::YM,
 SomaPiece::YM,SomaPiece::NO,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZP,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZP,SomaPiece::XM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::XP},
{SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::NO,SomaPiece::XM,SomaPiece::XP,SomaPiece::XP,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZP,SomaPiece::YM,SomaPiece::ZP,SomaPiece::YP,SomaPiece::ZM,SomaPiece::ZM},
{SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::XP,SomaPiece::NO,SomaPiece::XM,SomaPiece::XP,
 SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::ZP,SomaPiece::YP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP},
{SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::XP,SomaPiece::XP,SomaPiece::NO,SomaPiece::XM,
 SomaPiece::YM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::YM,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::YP},
{SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::XM,SomaPiece::XP,SomaPiece::XP,SomaPiece::NO,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZM,SomaPiece::ZP,
 SomaPiece::YM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::YP,SomaPiece::ZM},
{SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZP,
 SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZM,
 SomaPiece::NO,SomaPiece::XM,SomaPiece::XP,SomaPiece::XP,SomaPiece::ZP,SomaPiece::YP,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::YM},
{SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZM,
 SomaPiece::ZP,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::XP,SomaPiece::NO,SomaPiece::XM,SomaPiece::XP,SomaPiece::YP,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::ZP,SomaPiece::YM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP},
{SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,
 SomaPiece::ZP,SomaPiece::ZP,SomaPiece::YM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::XP,SomaPiece::XP,SomaPiece::NO,SomaPiece::XM,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZP,SomaPiece::YP,SomaPiece::ZP,SomaPiece::YM,SomaPiece::ZM,SomaPiece::ZM},
{SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZP,
 SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::YM,
 SomaPiece::XM,SomaPiece::XP,SomaPiece::XP,SomaPiece::NO,SomaPiece::ZM,SomaPiece::ZP,
 SomaPiece::YP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::YM,SomaPiece::ZM},
{SomaPiece::ZM,SomaPiece::ZM,SomaPiece::XM,SomaPiece::ZP,SomaPiece::XP,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZP,SomaPiece::YP,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::NO,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::ZP,SomaPiece::YM,SomaPiece::ZM,SomaPiece::XP,SomaPiece::ZM},
{SomaPiece::ZM,SomaPiece::XM,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::XP,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::YP,SomaPiece::ZM,
 SomaPiece::YM,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::NO,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::XP,SomaPiece::ZM,SomaPiece::YM},
{SomaPiece::XM,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,
 SomaPiece::XP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::YP,
 SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZM,SomaPiece::ZP,
 SomaPiece::NO,SomaPiece::ZM,SomaPiece::XP,SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZM},
{SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::XM,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZP,SomaPiece::XP,SomaPiece::YP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZP,SomaPiece::NO,SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZM,SomaPiece::XP},
{SomaPiece::XP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::XM,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZP,SomaPiece::ZM,
 SomaPiece::ZP,SomaPiece::YP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZM,
 SomaPiece::XP,SomaPiece::ZM,SomaPiece::NO,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP},
{SomaPiece::ZP,SomaPiece::XP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::XM,
 SomaPiece::ZP,SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::ZP,SomaPiece::YP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::XP,
 SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZP,SomaPiece::NO,SomaPiece::ZM,SomaPiece::ZM},
{SomaPiece::ZM,SomaPiece::ZP,SomaPiece::XP,SomaPiece::ZM,SomaPiece::XM,SomaPiece::ZP,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::YM,
 SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::YP,SomaPiece::XP,SomaPiece::ZM,
 SomaPiece::YM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::NO,SomaPiece::ZM},
{SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::XP,SomaPiece::ZP,SomaPiece::ZM,
 SomaPiece::ZM,SomaPiece::XM,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::YM,SomaPiece::ZP,
 SomaPiece::YP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::ZM,SomaPiece::YM,
 SomaPiece::ZM,SomaPiece::XP,SomaPiece::ZM,SomaPiece::ZM,SomaPiece::ZP,SomaPiece::NO}
};

//**************************************************************************

SomaPiece::SomaPiece(){
  curr_or = 0;
  nob = 1;
  blocks = new std::vector<SomaVector<int> >(nob);
  
  for(unsigned int i=0;i<nob;i++){
    (*blocks)[i] = SomaVector<int>(i,0,0);
  }  
}

//**************************************************************************

SomaPiece::SomaPiece(const unsigned int nobin){
#ifdef SOMA_DEBUG
  if(nobin == 0){
    std::cerr << "Attempt to create piece with no "
              << "blocks in SomaPiece::SomaPiece()." << std::endl;
  }
#endif
  curr_or = 0;
  nob = nobin;
  blocks = new std::vector<SomaVector<int> >(nob);
  
  for(unsigned int i=0;i<nob;i++){
    (*blocks)[i] = SomaVector<int>(i,0,0);
  }
}

//**************************************************************************

SomaPiece::SomaPiece(const SomaPiece& other){
  curr_or = other.curr_or;
  nob = other.nob;
  blocks = new std::vector<SomaVector<int> >(*other.blocks);
}

//**************************************************************************

SomaPiece::~SomaPiece(){
  delete blocks;
}

//**************************************************************************

void SomaPiece::set(const SomaPiece& other){

  if(this == &other) return;

  delete blocks;  

  curr_or = other.curr_or;
  nob = other.nob;

  blocks = new std::vector<SomaVector<int> >(*other.blocks);
}

//**************************************************************************

void SomaPiece::set(const unsigned int block,const SomaVector<int>& xyz){
#ifdef SOMA_DEBUG
  if(block >= nob){
    std::cerr << "Array bounds violated in SomaPiece::set()." << std::endl;
    return;
  }
#endif
  
  (*blocks)[block] = xyz;
}

//**************************************************************************

void SomaPiece::get(const unsigned int block,SomaVector<int> *const xyz)const{
#ifdef SOMA_DEBUG
  if(block >= nob){
    std::cerr << "Array bounds violated in SomaPiece::get()." << std::endl;
    return;
  }
#endif
  
  (*xyz) = (*blocks)[block];
}

//************************************************************************** 

void SomaPiece::set_orientation(const unsigned int orientation){
#ifdef SOMA_DEBUG
  if(orientation >= ORIENTATIONS){
    std::cerr << "Wrong orientation in SomaPiece::set_orientation()." 
              << std::endl;
    return;
  }
#endif
  
  while(curr_or != orientation)  
    rotate(set_pos_lookup[curr_or][orientation]);
}

//**************************************************************************

SomaVector<double> SomaPiece::get_offset()const{
  SomaVector<int> min((*blocks)[0]);
  SomaVector<int> max((*blocks)[0]);
  
  for(unsigned int i=1;i<nob;i++){
    if(max.x < (*blocks)[i].x) max.x = (*blocks)[i].x;
    if(max.y < (*blocks)[i].y) max.y = (*blocks)[i].y;
    if(max.z < (*blocks)[i].z) max.z = (*blocks)[i].z;

    if(min.x > (*blocks)[i].x) min.x = (*blocks)[i].x;
    if(min.y > (*blocks)[i].y) min.y = (*blocks)[i].y;
    if(min.z > (*blocks)[i].z) min.z = (*blocks)[i].z;    
  }
  
  SomaVector<double> retval(min+max);
  retval /= 2;
  retval.negate();
  
  return retval;
}

//**************************************************************************
  
void SomaPiece::rotate(const SomaPiece::DIRECTION direction){
  static int swt;
  
  switch (direction){
    case SomaPiece::ZM:
    for(unsigned int i=0;i<nob;i++){
      swt = (*blocks)[i].x;
      (*blocks)[i].x = (*blocks)[i].y;
      (*blocks)[i].y = -swt;
    }
    break;
    case SomaPiece::ZP:
    for(unsigned int i=0;i<nob;i++){
      swt = (*blocks)[i].y;
      (*blocks)[i].y = (*blocks)[i].x;
      (*blocks)[i].x= -swt;
    }   
    break;
    case SomaPiece::XP:
    for(unsigned int i=0;i<nob;i++){
      swt = (*blocks)[i].z;
      (*blocks)[i].z = (*blocks)[i].y;
      (*blocks)[i].y = -swt;
    }
    break;
    case SomaPiece::XM:
    for(unsigned int i=0;i<nob;i++){
      swt = (*blocks)[i].y;
      (*blocks)[i].y = (*blocks)[i].z;
      (*blocks)[i].z = -swt;
    }
    break;
    case SomaPiece::YP:
    for(unsigned int i=0;i<nob;i++){
      swt = (*blocks)[i].x;
      (*blocks)[i].x = (*blocks)[i].z;
      (*blocks)[i].z = -swt;
    }   
    break;
    case SomaPiece::YM:
    for(unsigned int i=0;i<nob;i++){
      swt = (*blocks)[i].z;
      (*blocks)[i].z = (*blocks)[i].x;
      (*blocks)[i].x = -swt;
    }
    break;
    default:
      std::cerr << "Unknown direction in SomaPiece::rotate()." << std::endl;
      return;
  }
  
  curr_or = lookup[curr_or][direction];
}

//**************************************************************************

bool SomaPiece::sticky(std::vector<SomaVector<int> >* newblocks, 
                      unsigned int newnob){
  std::vector<bool> found(newnob);
  std::vector<SomaVector<int> > queue(newnob);
  SomaVector<int> xp,xm,yp,ym,zp,zm;
  unsigned int qs = 0;
  unsigned int qe = 0;
  
  for (unsigned int i=0;i<newnob;i++)
    found[i] = false;
    
    queue[qs] = (*newblocks)[0];
    found[0] = true;
    qs++;
    
    while(qe != qs){    
      // get from queue
      xp = queue[qe];
      xp.add(1,0,0);
      xm = queue[qe];
      xm.subtract(1,0,0);
      
      yp = queue[qe];
      yp.add(0,1,0);
      ym = queue[qe];
      ym.subtract(0,1,0);
      
      zp = queue[qe];
      zp.add(0,0,1);
      zm = queue[qe];
      zm.subtract(0,0,1);
      
      qe++;
      
      //check six directions
      for(unsigned int i=0;i<newnob;i++){
        if(!found[i]){
          
          if(xp == (*newblocks)[i]){
            queue[qs] = (*newblocks)[i];
            found[i] = true;
            qs++;
          }else if(xm == (*newblocks)[i]){
            queue[qs] = (*newblocks)[i];
            found[i] = true;
            qs++;
          }else if(yp == (*newblocks)[i]){
            queue[qs] = (*newblocks)[i];
            found[i] = true;
            qs++;
          }else if(ym == (*newblocks)[i]){
            queue[qs] = (*newblocks)[i];
            found[i] = true;
            qs++;
          }else if(zp == (*newblocks)[i]){
            queue[qs] = (*newblocks)[i];
            found[i] = true;
            qs++;
          }else if(zm == (*newblocks)[i]){
            queue[qs] = (*newblocks)[i];
            found[i] = true;
            qs++;  
          }
          
        }
      }
    }
    
    for(unsigned int i=0;i<newnob;i++){
      if(!found[i]) return false;
    }
    return true;
}

//**************************************************************************

void SomaPiece::write(std::ostream& strm)const{
  strm << " { (";
  strm.width(3);
  strm << nob << ") ";
  for (unsigned int i=0;i<nob;i++){
    strm << (*blocks)[i];
  }
  strm << " } " << std::endl;
}

//**************************************************************************

bool SomaPiece::read(std::istream& strm){
  char S;
  bool error = false;
  unsigned int newnob=0;
  std::vector<SomaVector<int> >* newblocks=0;
  const SomaVector<int> zero;
  
  
  S=' '; 
  strm >> S;
  
  if(S!='{'){
    std::cerr << "Error in SomaPiece::read(). '{' expected." 
              << std::endl;
    error = true;
    goto error_stop;
  }
  
  
  S=' '; 
  strm >> S;
  
  if(S=='('){
    
    strm >> newnob;
    
    if(newnob>0){
      newblocks = new std::vector<SomaVector<int> >(newnob);
    }else{
      std::cerr << "Number of blocks 0 in SomaPiece::read()." << std::endl;
      error = true;
      goto error_stop;
    }
  }else{
    std::cerr << "Error in SomaPiece::read(). '(' expected." 
              << std::endl;
    error = true;
    goto error_stop;
  }
  
  S=' '; 
  strm >> S;
  
  if(S!=')'){
    std::cerr << "Error in SomaPiece::read(). ')' expected." 
              << std::endl;
    error = true;
    goto error_stop;
  }
  
  
  for(unsigned int i=0;i<newnob;i++){
    if(!(*newblocks)[i].read(strm)){
      error = true;
      goto error_stop;
    }
  }
  
  S=' '; 
  strm >> S;
  
  if(S!='}'){
    std::cerr << "Error in SomaPiece::read(). '}' expected." 
              << std::endl;
    error = true;
    goto error_stop;
  }
      
  // no double blocks
  for(unsigned int i=0;i<newnob;i++){
    for(unsigned int j=i+1;j<newnob;j++){
      if((*newblocks)[i] == (*newblocks)[j]){
        std::cerr << "Input semantically incorrect in SomaPiece::read()." 
                  << std::endl;
        std::cerr << "No double blocks allowed." << std::endl;
        error = true;
        goto error_stop;
      }
    }
  }
  
  // check blocks for connectedness !
  if(!sticky(newblocks,newnob)){ 
    std::cerr << "Input semantically incorrect in SomaPiece::read()." 
              << std::endl;
    std::cerr << "Blocks in one piece have to be connected." << std::endl;
    error = true;
    goto error_stop;
  }
  
  // ( 0 , 0 , 0 ) block required
  for(unsigned int i=0;i<newnob;i++){
    if((*newblocks)[i] == zero) goto error_stop;
  }
  std::cerr << "Input sematically incorrect in SomaPiece::read()." << std::endl;
  std::cerr << "( 0 , 0 , 0 ) block required." << std::endl;
  error = true;
  
  error_stop:
  
  if(error){
    
    if(newblocks) delete newblocks;
    return false;
    
  }else{
    
    delete blocks;
    
    nob = newnob;
    blocks = newblocks;
    curr_or = 0;
    
    return true;
  }
}

//**************************************************************************

