/*  Somatic - A soma puzzle solver.
 *  Copyright (C) 2003 Marc Moerig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef soma_piece
#define soma_piece

#include <vector>

#include "s_io.hpp"
#include "s_vector.hpp"

//**************************************************************************

/** A class representing our pieces. */
class SomaPiece : public SomaIO{
private:
  std::vector<SomaVector<int> >* blocks;
  unsigned int nob;
  unsigned int curr_or;
  
  bool sticky(std::vector<SomaVector<int> >* newblocks, unsigned int newnob);
public:

  enum DIRECTION {ZM=0,ZP,XP,XM,YM,YP,NO};
  static const unsigned int ORIENTATIONS;
  
  /** Create a new piece with one block */
  SomaPiece();
  
  /** Create a new piece for a given number of blocks. */
  SomaPiece(const unsigned int nobin);
  
  /** Create a new piece as copy of another piece.  */
  SomaPiece(const SomaPiece& other);
  
  /** Delete this piece. */
  ~SomaPiece();
  
  /** Get the number of blocks in this coordinate. */
  inline unsigned int get_num_blocks()const{return nob;}
  
  /** Set this pieces values to those from another piece. */
  void set(const SomaPiece& xyz);
  
  /** Change coordinates for a given block. */
  void set(const unsigned int block,const SomaVector<int>& xyz);

  /** Get coordiantes for a given block. */
  void get(const unsigned int block,SomaVector<int> *const xyz)const;

  /** Rotate a block into a given direction. */
  void rotate(const SomaPiece::DIRECTION direction);
  
  /** Set the orientation for this piece. */
  void set_orientation(const unsigned int orientation);
  
  /** get offset to center piece exactly */
  SomaVector<double> get_offset()const;
  
  /** Write this object's values to a stream. */
  void write(std::ostream& strm = std::cout)const;
  
  /** Read object values from a stream. Returns true if
  the object could be read successfully. */
  bool read(std::istream& strm = std::cin); 
  
};

//**************************************************************************

#endif

